@@ -133,6 +133,10 @@ class CoordinateManager:
         if coordinate_map_type == CoordinateMapType.CPU:
             postfix = "CPU"
         else:
+            assert (
+                _C.is_cuda_available()
+            ), "The MinkowskiEngine was compiled with CPU_ONLY flag. If you want to compile with CUDA support, make sure `torch.cuda.is_available()` is True when you install MinkowskiEngine."
+
             postfix = "GPU" + (
                 "_default" if allocator_type == GPUMemoryAllocatorType.CUDA else "_c10"
             )
@@ -158,6 +162,9 @@ class CoordinateManager:
         coordinate_map_type == `CoordinateMapType.GPU`) that defines the
         coordinates.
 
+        :attr:`tensor_stride` (`list`): a list of `D` elements that defines the
+        tensor stride for the new order-`D + 1` sparse tensor.
+
         Example::
 
            >>> manager = CoordinateManager(D=1)
@@ -174,7 +181,7 @@ class CoordinateManager:
     def insert_field(
         self,
         coordinates: torch.Tensor,
-        tensor_stride: Union[int, Sequence, np.ndarray],
+        tensor_stride: Sequence,
         string_id: str = "",
     ) -> Tuple[CoordinateMapKey, Tuple[torch.IntTensor, torch.IntTensor]]:
         r"""create a new coordinate map and returns
@@ -182,6 +189,10 @@ class CoordinateManager:
         :attr:`coordinates`: `torch.FloatTensor` (`CUDA` if coordinate_map_type
         == `CoordinateMapType.GPU`) that defines the coordinates.
 
+        :attr:`tensor_stride` (`list`): a list of `D` elements that defines the
+        tensor stride for the new order-`D + 1` sparse tensor.
+
+
         Example::
 
            >>> manager = CoordinateManager(D=1)
@@ -194,6 +205,52 @@ class CoordinateManager:
         """
         return self._manager.insert_field(coordinates, tensor_stride, string_id)
 
+    def field_to_sparse_insert_and_map(
+        self,
+        field_map_key: CoordinateMapKey,
+        sparse_tensor_stride: Union[int, Sequence, np.ndarray],
+        sparse_tensor_string_id: str = "",
+    ) -> Tuple[CoordinateMapKey, Tuple[torch.IntTensor, torch.IntTensor]]:
+
+        r"""Create a sparse tensor coordinate map with the tensor stride.
+
+        :attr:`field_map_key` (`CoordinateMapKey`): field map that a new sparse
+        tensor will be created from.
+
+        :attr:`tensor_stride` (`list`): a list of `D` elements that defines the
+        tensor stride for the new order-`D + 1` sparse tensor.
+
+        :attr:`string_id` (`str`): string id of the new sparse tensor coordinate map key.
+
+        Example::
+
+           >>> manager = CoordinateManager(D=1)
+           >>> coordinates = torch.FloatTensor([[0, 0.1], [0, 2.3], [0, 1.2], [0, 2.4]])
+           >>> key, (unique_map, inverse_map) = manager.insert(coordinates, [1])
+
+        """
+        return self._manager.field_to_sparse_insert_and_map(
+            field_map_key, sparse_tensor_stride, sparse_tensor_string_id
+        )
+
+    def exists_field_to_sparse(
+        self, field_map_key: CoordinateMapKey, sparse_map_key: CoordinateMapKey
+    ):
+        return self._manager.exists_field_to_sparse(field_map_key, sparse_map_key)
+
+    def field_to_sparse_keys(self, field_map_key: CoordinateMapKey):
+        return self._manager.field_to_sparse_keys(field_map_key.get_key())
+
+    def get_field_to_sparse_map(
+        self, field_map_key: CoordinateMapKey, sparse_map_key: CoordinateMapKey
+    ):
+        return self._manager.get_field_to_sparse_map(field_map_key, sparse_map_key)
+
+    def field_to_sparse_map(
+        self, field_map_key: CoordinateMapKey, sparse_map_key: CoordinateMapKey
+    ):
+        return self._manager.field_to_sparse_map(field_map_key, sparse_map_key)
+
     def stride(
         self,
         coordinate_map_key: CoordinateMapKey,
@@ -213,6 +270,9 @@ class CoordinateManager:
     def origin(self) -> CoordinateMapKey:
         return self._manager.origin()
 
+    def origin_field(self) -> CoordinateMapKey:
+        return self._manager.origin_field()
+
     def size(self, coordinate_map_key: CoordinateMapKey) -> int:
         return self._manager.size(coordinate_map_key)
 
@@ -280,6 +340,9 @@ class CoordinateManager:
         """
         Returns a unique coordinate_map_key for a given tensor stride.
 
+        :attr:`tensor_stride` (`list`): a list of `D` elements that defines the
+        tensor stride for the new order-`D + 1` sparse tensor.
+
         """
         return self._manager.get_random_string_id(tensor_stride, "")
 
@@ -294,6 +357,34 @@ class CoordinateManager:
         region_offset=None,
         is_transpose=False,
         is_pool=False,
+    ) -> dict:
+        r"""Alias of :attr:`CoordinateManager.kernel_map`. Will be deprecated in the next version."""
+        warnings.warn(
+            "`get_kernel_map` will be deprecated. Please use `kernel_map` instead."
+        )
+        return self.kernel_map(
+            in_key,
+            out_key,
+            stride,
+            kernel_size,
+            dilation,
+            region_type,
+            region_offset,
+            is_transpose,
+            is_pool,
+        )
+
+    def kernel_map(
+        self,
+        in_key: CoordinateMapKey,
+        out_key: CoordinateMapKey,
+        stride=1,
+        kernel_size=3,
+        dilation=1,
+        region_type=RegionType.HYPER_CUBE,
+        region_offset=None,
+        is_transpose=False,
+        is_pool=False,
     ) -> dict:
         r"""Get kernel in-out maps for the specified coords keys or tensor strides.
 
@@ -315,11 +406,11 @@ class CoordinateManager:
         if region_offset is None:
             region_offset = torch.IntTensor()
 
-        kernel_map = self._manager.get_kernel_map(
+        kernel_map = self._manager.kernel_map(
             in_key,
             out_key,
-            convert_to_int_list(stride, self.D),  #
             convert_to_int_list(kernel_size, self.D),  #
+            convert_to_int_list(stride, self.D),  #
             convert_to_int_list(dilation, self.D),  #
             region_type,
             region_offset,
@@ -332,6 +423,12 @@ class CoordinateManager:
     def origin_map(self, key: CoordinateMapKey):
         return self._manager.origin_map(key)
 
+    def origin_field_map(self, key: CoordinateMapKey):
+        return self._manager.origin_field_map(key)
+
+    def stride_map(self, in_key: CoordinateMapKey, stride_key: CoordinateMapKey):
+        return self._manager.stride_map(in_key, stride_key)
+
     def union_map(self, in_keys: list, out_key):
         return self._manager.union_map(in_keys, out_key)
 
