@@ -1,4 +1,5 @@
-# Copyright (c) Chris Choy (chrischoy@ai.stanford.edu).
+# Copyright (c) 2021 NVIDIA CORPORATION.
+# Copyright (c) 2018-2020 Chris Choy (chrischoy@ai.stanford.edu).
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy of
 # this software and associated documentation files (the "Software"), to deal in
@@ -66,23 +67,11 @@ class MinkowskiLinear(Module):
         return self.__class__.__name__ + s
 
 
-def cat(*sparse_tensors):
-    r"""Concatenate sparse tensors
+def _tuple_operator(*sparse_tensors, operator):
+    if len(sparse_tensors) == 1:
+        assert isinstance(sparse_tensors[0], (tuple, list))
+        sparse_tensors = sparse_tensors[0]
 
-    Concatenate sparse tensor features. All sparse tensors must have the same
-    `coords_key` (the same coordinates). To concatenate sparse tensors with
-    different sparsity patterns, use SparseTensor binary operations, or
-    :attr:`MinkowskiEngine.MinkowskiUnion`.
-
-    Example::
-
-       >>> import MinkowskiEngine as ME
-       >>> sin = ME.SparseTensor(feats, coords)
-       >>> sin2 = ME.SparseTensor(feats2, coordinate_map_key=sin.coordinate_map_key, coords_man=sin.coordinate_manager)
-       >>> sout = UNet(sin)  # Returns an output sparse tensor on the same coordinates
-       >>> sout2 = ME.cat(sin, sin2, sout)  # Can concatenate multiple sparse tensors
-
-    """
     assert (
         len(sparse_tensors) > 1
     ), f"Invalid number of inputs. The input must be at least two len(sparse_tensors) > 1"
@@ -111,7 +100,7 @@ def cat(*sparse_tensors):
         for s in sparse_tensors:
             tens.append(s.F)
         return SparseTensor(
-            torch.cat(tens, dim=1),
+            operator(tens),
             coordinate_map_key=coordinate_map_key,
             coordinate_manager=coordinate_manager,
         )
@@ -139,7 +128,7 @@ def cat(*sparse_tensors):
         for s in sparse_tensors:
             tens.append(s.F)
         return TensorField(
-            torch.cat(tens, dim=1),
+            operator(tens),
             coordinate_field_map_key=coordinate_field_map_key,
             coordinate_manager=coordinate_manager,
         )
@@ -149,6 +138,111 @@ def cat(*sparse_tensors):
         )
 
 
+def cat(*sparse_tensors):
+    r"""Concatenate sparse tensors
+
+    Concatenate sparse tensor features. All sparse tensors must have the same
+    `coordinate_map_key` (the same coordinates). To concatenate sparse tensors
+    with different sparsity patterns, use SparseTensor binary operations, or
+    :attr:`MinkowskiEngine.MinkowskiUnion`.
+
+    Example::
+
+       >>> import MinkowskiEngine as ME
+       >>> sin = ME.SparseTensor(feats, coords)
+       >>> sin2 = ME.SparseTensor(feats2, coordinate_map_key=sin.coordinate_map_key, coordinate_mananger=sin.coordinate_manager)
+       >>> sout = UNet(sin)  # Returns an output sparse tensor on the same coordinates
+       >>> sout2 = ME.cat(sin, sin2, sout)  # Can concatenate multiple sparse tensors
+
+    """
+    return _tuple_operator(*sparse_tensors, operator=lambda xs: torch.cat(xs, dim=-1))
+
+
+def _sum(*sparse_tensors):
+    r"""Compute the sum of sparse tensor features
+
+    Sum all sparse tensor features. All sparse tensors must have the same
+    `coordinate_map_key` (the same coordinates). To sum sparse tensors with
+    different sparsity patterns, use SparseTensor binary operations, or
+    :attr:`MinkowskiEngine.MinkowskiUnion`.
+
+    Example::
+
+       >>> import MinkowskiEngine as ME
+       >>> sin = ME.SparseTensor(feats, coords)
+       >>> sin2 = ME.SparseTensor(feats2, coordinate_map_key=sin.coordinate_map_key, coordinate_manager=sin.coordinate_manager)
+       >>> sout = UNet(sin)  # Returns an output sparse tensor on the same coordinates
+       >>> sout2 = ME.sum(sin, sin2, sout)  # Can concatenate multiple sparse tensors
+
+    """
+
+    def return_sum(xs):
+        tmp = xs[0] + xs[1]
+        for x in xs[2:]:
+            tmp += x
+        return tmp
+
+    return _tuple_operator(*sparse_tensors, operator=lambda xs: return_sum(xs))
+
+
+def mean(*sparse_tensors):
+    r"""Compute the average of sparse tensor features
+
+    Sum all sparse tensor features. All sparse tensors must have the same
+    `coordinate_map_key` (the same coordinates). To sum sparse tensors with
+    different sparsity patterns, use SparseTensor binary operations, or
+    :attr:`MinkowskiEngine.MinkowskiUnion`.
+
+    Example::
+
+       >>> import MinkowskiEngine as ME
+       >>> sin = ME.SparseTensor(feats, coords)
+       >>> sin2 = ME.SparseTensor(feats2, coordinate_map_key=sin.coordinate_map_key, coordinate_manager=sin.coordinate_manager)
+       >>> sout = UNet(sin)  # Returns an output sparse tensor on the same coordinates
+       >>> sout2 = ME.mean(sin, sin2, sout)  # Can concatenate multiple sparse tensors
+
+    """
+
+    def return_mean(xs):
+        tmp = xs[0] + xs[1]
+        for x in xs[2:]:
+            tmp += x
+        return tmp / len(xs)
+
+    return _tuple_operator(*sparse_tensors, operator=lambda xs: return_mean(xs))
+
+
+def var(*sparse_tensors):
+    r"""Compute the variance of sparse tensor features
+
+    Sum all sparse tensor features. All sparse tensors must have the same
+    `coordinate_map_key` (the same coordinates). To sum sparse tensors with
+    different sparsity patterns, use SparseTensor binary operations, or
+    :attr:`MinkowskiEngine.MinkowskiUnion`.
+
+    Example::
+
+       >>> import MinkowskiEngine as ME
+       >>> sin = ME.SparseTensor(feats, coords)
+       >>> sin2 = ME.SparseTensor(feats2, coordinate_map_key=sin.coordinate_map_key, coordinate_manager=sin.coordinate_manager)
+       >>> sout = UNet(sin)  # Returns an output sparse tensor on the same coordinates
+       >>> sout2 = ME.var(sin, sin2, sout)  # Can concatenate multiple sparse tensors
+
+    """
+
+    def return_var(xs):
+        tmp = xs[0] + xs[1]
+        for x in xs[2:]:
+            tmp += x
+        mean = tmp / len(xs)
+        var = (xs[0] - mean) ** 2
+        for x in xs[1:]:
+            var += (x - mean) ** 2
+        return var / len(xs)
+
+    return _tuple_operator(*sparse_tensors, operator=lambda xs: return_var(xs))
+
+
 def dense_coordinates(shape: Union[list, torch.Size]):
     """
     coordinates = dense_coordinates(tensor.shape)
@@ -182,8 +276,49 @@ def dense_coordinates(shape: Union[list, torch.Size]):
     return coordinates
 
 
-def to_sparse(dense_tensor: torch.Tensor, coordinates: torch.Tensor = None):
-    r"""Converts a (differentiable) dense tensor to a sparse tensor.
+def to_sparse(x: torch.Tensor, format: str = None, coordinates=None, device=None):
+    r"""Convert a batched tensor (dimension 0 is the batch dimension) to a SparseTensor
+
+    :attr:`x` (:attr:`torch.Tensor`): a batched tensor. The first dimension is the batch dimension.
+
+    :attr:`format` (:attr:`str`): Format of the tensor. It must include 'B' and 'C' indicating the batch and channel dimension respectively. The rest of the dimensions must be 'X'. .e.g. format="BCXX" if image data with BCHW format is used. If a 3D data with the channel at the last dimension, use format="BXXXC" indicating Batch X Height X Width X Depth X Channel. If not provided, the format will be "BCX...X".
+
+    :attr:`device`: Device the sparse tensor will be generated on. If not provided, the device of the input tensor will be used.
+
+    """
+    assert x.ndim > 2, "Input has 0 spatial dimension."
+    assert isinstance(x, torch.Tensor)
+    if format is None:
+        format = [
+            "X",
+        ] * x.ndim
+        format[0] = "B"
+        format[1] = "C"
+        format = "".join(format)
+    assert x.ndim == len(format), f"Invalid format: {format}. len(format) != x.ndim"
+    assert (
+        "B" in format and "B" == format[0] and format.count("B") == 1
+    ), "The input must have the batch axis and the format must include 'B' indicating the batch axis."
+    assert (
+        "C" in format and format.count("C") == 1
+    ), "The format must indicate the channel axis"
+    if device is None:
+        device = x.device
+    ch_dim = format.find("C")
+    reduced_x = torch.abs(x).sum(ch_dim)
+    bcoords = torch.where(reduced_x != 0)
+    stacked_bcoords = torch.stack(bcoords, dim=1).int()
+    indexing = [f"bcoords[{i}]" for i in range(len(bcoords))]
+    indexing.insert(ch_dim, ":")
+    features = torch.zeros(
+        (len(stacked_bcoords), x.size(ch_dim)), dtype=x.dtype, device=x.device
+    )
+    exec("features[:] = x[" + ", ".join(indexing) + "]")
+    return SparseTensor(features=features, coordinates=stacked_bcoords, device=device)
+
+
+def to_sparse_all(dense_tensor: torch.Tensor, coordinates: torch.Tensor = None):
+    r"""Converts a (differentiable) dense tensor to a sparse tensor with all coordinates.
 
     Assume the input to have BxCxD1xD2x....xDN format.
 
@@ -209,7 +344,7 @@ def to_sparse(dense_tensor: torch.Tensor, coordinates: torch.Tensor = None):
     return SparseTensor(
         feat_tensor.reshape(-1, dense_tensor.size(1)),
         coordinates,
-        device=dense_tensor.dtype,
+        device=dense_tensor.device,
     )
 
 
@@ -218,6 +353,9 @@ class MinkowskiToSparseTensor(MinkowskiModuleBase):
 
     For dense tensor, the input must have the BxCxD1xD2x....xDN format.
 
+    :attr:`remove_zeros` (bool): if True, removes zero valued coordinates. If
+    False, use all coordinates to populate a sparse tensor. True by default.
+
     If the shape of the tensor do not change, use `dense_coordinates` to cache the coordinates.
     Please refer to tests/python/dense.py for usage.
 
@@ -233,7 +371,7 @@ class MinkowskiToSparseTensor(MinkowskiModuleBase):
        >>> network = nn.Sequential(
        >>>     # Add layers that can be applied on a regular pytorch tensor
        >>>     nn.ReLU(),
-       >>>     MinkowskiToSparseTensor(coordinates=coordinates),
+       >>>     MinkowskiToSparseTensor(remove_zeros=False, coordinates=coordinates),
        >>>     MinkowskiConvolution(4, 5, kernel_size=3, dimension=4),
        >>>     MinkowskiBatchNorm(5),
        >>>     MinkowskiReLU(),
@@ -247,8 +385,12 @@ class MinkowskiToSparseTensor(MinkowskiModuleBase):
 
     """
 
-    def __init__(self, coordinates: torch.Tensor = None):
+    def __init__(self, remove_zeros=True, coordinates: torch.Tensor = None):
         MinkowskiModuleBase.__init__(self)
+        assert (
+            remove_zeros and coordinates is None
+        ), "The coordinates argument cannot be used with remove_zeros=True. If you want to use the coordinates argument, provide remove_zeros=False."
+        self.remove_zeros = remove_zeros
         self.coordinates = coordinates
 
     def forward(self, input: Union[TensorField, torch.Tensor]):
@@ -256,7 +398,10 @@ class MinkowskiToSparseTensor(MinkowskiModuleBase):
             return input.sparse()
         elif isinstance(input, torch.Tensor):
             # dense tensor to sparse tensor conversion
-            return to_sparse(input, self.coordinates)
+            if self.remove_zeros:
+                return to_sparse(input)
+            else:
+                return to_sparse_all(input, self.coordinates)
         else:
             raise ValueError(
                 "Unsupported type. Only TensorField and torch.Tensor are supported"
@@ -310,3 +455,23 @@ class MinkowskiToDenseTensor(MinkowskiModuleBase):
 
     def __repr__(self):
         return self.__class__.__name__ + "()"
+
+
+class MinkowskiStackCat(torch.nn.Sequential):
+    def forward(self, x):
+        return cat([module(x) for module in self])
+
+
+class MinkowskiStackSum(torch.nn.Sequential):
+    def forward(self, x):
+        return _sum([module(x) for module in self])
+
+
+class MinkowskiStackMean(torch.nn.Sequential):
+    def forward(self, x):
+        return mean([module(x) for module in self])
+
+
+class MinkowskiStackVar(torch.nn.Sequential):
+    def forward(self, x):
+        return var([module(x) for module in self])
