@@ -27,6 +27,7 @@ from typing import Union
 import torch
 from torch.autograd import Function
 
+import MinkowskiEngineBackend._C as _C
 from MinkowskiEngineBackend._C import CoordinateMapKey, PoolingMode
 from MinkowskiSparseTensor import SparseTensor, _get_coordinate_map_key
 from MinkowskiCoordinateManager import CoordinateManager
@@ -35,6 +36,7 @@ from MinkowskiCommon import (
     MinkowskiModuleBase,
     get_minkowski_function,
 )
+import MinkowskiEngine as ME
 
 
 class MinkowskiLocalPoolingFunction(Function):
@@ -129,7 +131,9 @@ class MinkowskiPoolingBase(MinkowskiModuleBase):
         dimension=-1,
     ):
         super(MinkowskiPoolingBase, self).__init__()
-        assert dimension > 0, f"dimension must be a positive integer, {dimension}"
+        assert (
+            dimension > 0
+        ), f"Invalid dimension. Please provide a valid dimension argument. dimension={dimension}"
 
         if kernel_generator is None:
             kernel_generator = KernelGenerator(
@@ -606,9 +610,7 @@ class MinkowskiGlobalPoolingFunction(Function):
             ctx.out_coords_key,
             ctx.coordinate_manager._manager,
         )
-
         ctx.num_nonzero = num_nonzero
-
         return out_feat
 
     @staticmethod
@@ -704,7 +706,7 @@ class MinkowskiGlobalMaxPooling(MinkowskiGlobalPooling):
 
     .. math::
 
-        \mathbf{y} = \frac{1}{|\mathcal{C}^\text{in}|} \max_{\mathbf{i} \in
+        \mathbf{y} = \max_{\mathbf{i} \in
         \mathcal{C}^\text{in}} \mathbf{x}_{\mathbf{i}}
 
     """
@@ -717,3 +719,62 @@ class MinkowskiGlobalMaxPooling(MinkowskiGlobalPooling):
 
         """
         MinkowskiGlobalPooling.__init__(self, mode=mode)
+
+    def forward(
+        self,
+        input,
+        coordinates: Union[torch.IntTensor, CoordinateMapKey, SparseTensor] = None,
+    ):
+        # Get a new coordinate map key or extract one from the coordinates
+        if isinstance(input, ME.TensorField):
+            in_coordinate_map_key = input.coordinate_field_map_key
+            out_coordinate_map_key = CoordinateMapKey(
+                input.coordinate_field_map_key.get_coordinate_size()
+            )
+        else:
+            in_coordinate_map_key = input.coordinate_map_key
+            out_coordinate_map_key = _get_coordinate_map_key(input, coordinates)
+        output = self.pooling.apply(
+            input.F,
+            self.pooling_mode,
+            in_coordinate_map_key,
+            out_coordinate_map_key,
+            input._manager,
+        )
+
+        return SparseTensor(
+            output,
+            coordinate_map_key=out_coordinate_map_key,
+            coordinate_manager=input.coordinate_manager,
+        )
+
+
+class MinkowskiDirectMaxPoolingFunction(Function):
+    @staticmethod
+    def forward(
+        ctx,
+        in_map: torch.Tensor,
+        out_map: torch.Tensor,
+        in_feat: torch.Tensor,
+        out_nrows: int,
+        is_sorted: bool = False,
+    ):
+        out_feat, max_mask = _C.direct_max_pool_fw(
+            in_map, out_map, in_feat, out_nrows, is_sorted
+        )
+        ctx.in_nrows = in_feat.size(0)
+        ctx.save_for_backward(max_mask)
+        return out_feat
+
+    @staticmethod
+    def backward(ctx, grad_out_feat):
+        grad_out_feat = grad_out_feat.contiguous()
+        max_mask = ctx.saved_tensors[0]
+        grad = _C.direct_max_pool_bw(grad_out_feat, max_mask, ctx.in_nrows)
+        return (
+            None,
+            None,
+            grad,
+            None,
+            None,
+        )
