@@ -42,7 +42,7 @@ from MinkowskiTensor import (
     set_global_coordinate_manager,
 )
 from MinkowskiCoordinateManager import CoordinateManager
-from sparse_matrix_functions import MinkowskiSPMMFunction
+from sparse_matrix_functions import MinkowskiSPMMFunction, MinkowskiSPMMAverageFunction
 
 
 class SparseTensor(Tensor):
@@ -296,10 +296,7 @@ class SparseTensor(Tensor):
         self.unique_index = unique_index.long()
         coordinates = coordinates[self.unique_index]
 
-        if self.quantization_mode in [
-            SparseTensorQuantizationMode.UNWEIGHTED_SUM,
-            SparseTensorQuantizationMode.UNWEIGHTED_AVERAGE,
-        ]:
+        if self.quantization_mode == SparseTensorQuantizationMode.UNWEIGHTED_SUM:
             spmm = MinkowskiSPMMFunction()
             N = len(features)
             cols = torch.arange(
@@ -310,18 +307,16 @@ class SparseTensor(Tensor):
             vals = torch.ones(N, dtype=features.dtype, device=features.device)
             size = torch.Size([len(self.unique_index), len(self.inverse_mapping)])
             features = spmm.apply(self.inverse_mapping, cols, vals, size, features)
-            if (
-                self.quantization_mode
-                == SparseTensorQuantizationMode.UNWEIGHTED_AVERAGE
-            ):
-                nums = spmm.apply(
-                    self.inverse_mapping,
-                    cols,
-                    vals,
-                    size,
-                    vals.reshape(N, 1),
-                )
-                features /= nums
+        elif self.quantization_mode == SparseTensorQuantizationMode.UNWEIGHTED_AVERAGE:
+            spmm_avg = MinkowskiSPMMAverageFunction()
+            N = len(features)
+            cols = torch.arange(
+                N,
+                dtype=self.inverse_mapping.dtype,
+                device=self.inverse_mapping.device,
+            )
+            size = torch.Size([len(self.unique_index), len(self.inverse_mapping)])
+            features = spmm_avg.apply(self.inverse_mapping, cols, size, features)
         elif self.quantization_mode == SparseTensorQuantizationMode.RANDOM_SUBSAMPLE:
             features = features[self.unique_index]
         else:
@@ -491,7 +486,11 @@ class SparseTensor(Tensor):
         if min_coordinate is None:
             min_coordinate, _ = self.C.min(0, keepdim=True)
             min_coordinate = min_coordinate[:, 1:]
-            coords = self.C[:, 1:] - min_coordinate
+            if not torch.all(min_coordinate >= 0):
+                raise ValueError(
+                    f"Coordinate has a negative value: {min_coordinate}. Please provide min_coordinate argument"
+                )
+            coords = self.C[:, 1:]
         elif isinstance(min_coordinate, int) and min_coordinate == 0:
             coords = self.C[:, 1:]
         else:
@@ -564,7 +563,7 @@ class SparseTensor(Tensor):
 
         if isinstance(X, TensorField):
             return TensorField(
-                self.F[X.inverse_mapping],
+                self.F[X.inverse_mapping(self.coordinate_map_key).long()],
                 coordinate_field_map_key=X.coordinate_field_map_key,
                 coordinate_manager=X.coordinate_manager,
                 quantization_mode=X.quantization_mode,
@@ -616,7 +615,9 @@ class SparseTensor(Tensor):
 
         from MinkowskiTensorField import TensorField
 
-        features = torch.cat((self.F[X.inverse_mapping], X.F), dim=1)
+        features = torch.cat(
+            (self.F[X.inverse_mapping(self.coordinate_map_key)], X.F), dim=1
+        )
         if isinstance(X, TensorField):
             return TensorField(
                 features,
@@ -630,7 +631,7 @@ class SparseTensor(Tensor):
             ), "Slice can only be applied on the same coordinates (coordinate_map_key)"
             return TensorField(
                 features,
-                coordinates=self.C[X.inverse_mapping],
+                coordinates=self.C[X.inverse_mapping(self.coordinate_map_key)],
                 coordinate_manager=self.coordinate_manager,
                 quantization_mode=self.quantization_mode,
             )
