@@ -22,7 +22,7 @@
 # Please cite "4D Spatio-Temporal ConvNets: Minkowski Convolutional Neural
 # Networks", CVPR'19 (https://arxiv.org/abs/1904.08755) if you use any part
 # of the code.
-__version__ = "0.5.0"
+__version__ = "0.5.2"
 
 import os
 import sys
@@ -48,6 +48,8 @@ if os.cpu_count() > 16 and "OMP_NUM_THREADS" not in os.environ:
 # Must be imported first to load all required shared libs
 import torch
 
+from diagnostics import print_diagnostics
+
 from MinkowskiEngineBackend._C import (
     MinkowskiAlgorithm,
     CoordinateMapKey,
@@ -58,6 +60,7 @@ from MinkowskiEngineBackend._C import (
     BroadcastMode,
     is_cuda_available,
     cuda_version,
+    cudart_version,
     get_gpu_memory_info,
 )
 
@@ -117,6 +120,7 @@ from MinkowskiPooling import (
     MinkowskiGlobalSumPooling,
     MinkowskiGlobalAvgPooling,
     MinkowskiGlobalMaxPooling,
+    MinkowskiDirectMaxPoolingFunction,
 )
 
 from MinkowskiBroadcast import (
@@ -128,17 +132,34 @@ from MinkowskiBroadcast import (
 )
 
 from MinkowskiNonlinearity import (
-    MinkowskiReLU,
-    MinkowskiSigmoid,
-    MinkowskiSoftmax,
-    MinkowskiPReLU,
     MinkowskiELU,
+    MinkowskiHardshrink,
+    MinkowskiHardsigmoid,
+    MinkowskiHardtanh,
+    MinkowskiHardswish,
+    MinkowskiLeakyReLU,
+    MinkowskiLogSigmoid,
+    MinkowskiPReLU,
+    MinkowskiReLU,
+    MinkowskiReLU6,
+    MinkowskiRReLU,
     MinkowskiSELU,
     MinkowskiCELU,
     MinkowskiGELU,
-    MinkowskiDropout,
-    MinkowskiThreshold,
+    MinkowskiSigmoid,
+    MinkowskiSiLU,
+    MinkowskiSoftplus,
+    MinkowskiSoftshrink,
+    MinkowskiSoftsign,
     MinkowskiTanh,
+    MinkowskiTanhshrink,
+    MinkowskiThreshold,
+    MinkowskiSoftmin,
+    MinkowskiSoftmax,
+    MinkowskiLogSoftmax,
+    MinkowskiAdaptiveLogSoftmaxWithLoss,
+    MinkowskiDropout,
+    MinkowskiAlphaDropout,
     MinkowskiSinusoidal,
 )
 
@@ -168,15 +189,39 @@ from MinkowskiOps import (
     MinkowskiLinear,
     MinkowskiToSparseTensor,
     MinkowskiToDenseTensor,
+    MinkowskiStackCat,
+    MinkowskiStackSum,
+    MinkowskiStackMean,
+    MinkowskiStackVar,
     cat,
+    mean,
+    var,
     to_sparse,
+    to_sparse_all,
     dense_coordinates,
 )
 
+from MinkowskiOps import _sum as sum
+
 import MinkowskiFunctional
 
 import MinkowskiEngine.utils as utils
 
 import MinkowskiEngine.modules as modules
 
-from sparse_matrix_functions import spmm, MinkowskiSPMMFunction
+from sparse_matrix_functions import (
+    spmm,
+    MinkowskiSPMMFunction,
+    MinkowskiSPMMAverageFunction,
+)
+
+
+if not is_cuda_available():
+    warnings.warn(
+        " ".join(
+            [
+                "The MinkowskiEngine was compiled with CPU_ONLY flag.",
+                "If you want to compile with CUDA support, make sure `torch.cuda.is_available()` is True when you install MinkowskiEngine.",
+            ]
+        )
+    )
