@@ -35,6 +35,7 @@
 #ifndef CPU_ONLY
 #include "allocators.cuh"
 #include "coordinate_map_gpu.cuh"
+#include <cuda.h>
 #endif
 
 #include <torch/extension.h>
@@ -488,12 +489,28 @@ at::Tensor quantization_average_features(at::Tensor in_feat, at::Tensor in_map,
                                          int mode);
 */
 
+std::pair<torch::Tensor, torch::Tensor>
+max_pool_fw(torch::Tensor const &in_map,  //
+            torch::Tensor const &out_map, //
+            torch::Tensor const &in_feat, //
+            int const out_nrows, bool const is_sorted);
+
+torch::Tensor max_pool_bw(torch::Tensor const &grad_out_feat, //
+                          torch::Tensor const &mask_index,    //
+                          int const in_nrows);
+
 #ifndef CPU_ONLY
 template <typename th_int_type>
 torch::Tensor coo_spmm(torch::Tensor const &rows, torch::Tensor const &cols,
                        torch::Tensor const &vals, int64_t const dim_i,
                        int64_t const dim_j, torch::Tensor const &mat2,
-                       int64_t spmm_algorithm_id);
+                       int64_t const spmm_algorithm_id, bool const is_sorted);
+
+template <typename th_int_type>
+std::vector<torch::Tensor> // output, sorted rows, sorted cols, sorted vals.
+coo_spmm_average(torch::Tensor const &rows, torch::Tensor const &cols,
+                 int64_t const dim_i, int64_t const dim_j,
+                 torch::Tensor const &mat2, int64_t const spmm_algorithm_id);
 
 std::pair<size_t, size_t> get_memory_info();
 #endif
@@ -639,12 +656,18 @@ void instantiate_gpu_func(py::module &m, const std::string &dtypestr) {
 void non_templated_cpu_func(py::module &m) {
   m.def("quantize_np", &minkowski::quantize_np);
   m.def("quantize_th", &minkowski::quantize_th);
+  m.def("direct_max_pool_fw", &minkowski::max_pool_fw,
+        py::call_guard<py::gil_scoped_release>());
+  m.def("direct_max_pool_bw", &minkowski::max_pool_bw,
+        py::call_guard<py::gil_scoped_release>());
 }
 
 #ifndef CPU_ONLY
 void non_templated_gpu_func(py::module &m) {
   m.def("coo_spmm_int32", &minkowski::coo_spmm<int32_t>,
         py::call_guard<py::gil_scoped_release>());
+  m.def("coo_spmm_average_int32", &minkowski::coo_spmm_average<int32_t>,
+        py::call_guard<py::gil_scoped_release>());
 }
 #endif
 
@@ -727,6 +750,7 @@ void initialize_non_templated_classes(py::module &m) {
       .def(py::init<minkowski::default_types::size_type>())
       .def(py::init<minkowski::default_types::stride_type, std::string>())
       .def("__repr__", &minkowski::CoordinateMapKey::to_string)
+      .def("__hash__", &minkowski::CoordinateMapKey::hash)
       .def("is_key_set", &minkowski::CoordinateMapKey::is_key_set)
       .def("get_coordinate_size",
            &minkowski::CoordinateMapKey::get_coordinate_size)
@@ -755,18 +779,30 @@ void instantiate_manager(py::module &m, const std::string &dtypestr) {
                &manager_type::to_string, py::const_))
       .def("insert_and_map", &manager_type::insert_and_map)
       .def("insert_field", &manager_type::insert_field)
+      .def("field_to_sparse_map", &manager_type::field_to_sparse_map)
+      .def("field_to_sparse_insert_and_map",
+           &manager_type::field_to_sparse_insert_and_map)
+      .def("exists_field_to_sparse",
+           py::overload_cast<minkowski::CoordinateMapKey const *,
+                             minkowski::CoordinateMapKey const *>(
+               &manager_type::exists_field_to_sparse, py::const_))
+      .def("get_field_to_sparse_map", &manager_type::get_field_to_sparse_map)
       .def("stride", &manager_type::py_stride)
       .def("origin", &manager_type::py_origin)
+      .def("origin_field", &manager_type::py_origin_field)
       .def("get_coordinates", &manager_type::get_coordinates)
       .def("get_coordinate_field", &manager_type::get_coordinate_field)
       .def("get_coordinate_map_keys", &manager_type::get_coordinate_map_keys)
+      .def("field_to_sparse_keys", &manager_type::field_to_sparse_keys)
       .def("size", py::overload_cast<minkowski::CoordinateMapKey const *>(
                        &manager_type::size, py::const_))
       .def("get_random_string_id", &manager_type::get_random_string_id)
       .def("origin_map_size", &manager_type::origin_map_size)
       .def("origin_map", &manager_type::origin_map_th)
+      .def("origin_field_map", &manager_type::origin_field_map_th)
       .def("union_map", &manager_type::union_map_th)
-      .def("get_kernel_map", &manager_type::get_kernel_map)
+      .def("stride_map", &manager_type::stride_map_th)
+      .def("kernel_map", &manager_type::kernel_map_th)
       .def("interpolation_map_weight", &manager_type::interpolation_map_weight);
 }
 
@@ -779,6 +815,14 @@ bool is_cuda_available() {
 }
 
 int cuda_version() {
+#if defined(CUDA_VERSION)
+  return CUDA_VERSION;
+#else
+  return -1;
+#endif
+}
+
+int cudart_version() {
 #if defined(CUDART_VERSION)
   return CUDART_VERSION;
 #else
