@@ -45,93 +45,6 @@ default_types::stride_type zeros(size_t const len) { return _fill_vec<0>(len); }
 default_types::stride_type ones(size_t const len) { return _fill_vec<1>(len); }
 
 } // namespace detail
-/*
-
-template <typename MapType>
-vector<at::Tensor>
-CoordsManager<MapType>::getCoordsMap(py::object py_in_coords_key,
-                                     py::object py_out_coords_key) const {
-  CoordsKey *p_in_coords_key = py_in_coords_key.cast<CoordsKey *>();
-  CoordsKey *p_out_coords_key = py_out_coords_key.cast<CoordsKey *>();
-  const uint64_t in_coords_key = p_in_coords_key->getKey();
-  const uint64_t out_coords_key = p_out_coords_key->getKey();
-
-  const auto in_map_iter = coords_maps.find(in_coords_key);
-  const auto out_map_iter = coords_maps.find(out_coords_key);
-
-  ASSERT(in_map_iter != coords_maps.end(), "Input coords not found at",
-         to_string(in_coords_key));
-  ASSERT(out_map_iter != coords_maps.end(), "Output coords not found at",
-         to_string(out_coords_key));
-
-  const auto &out_tensor_strides = p_out_coords_key->getTensorStride();
-  const auto in_out =
-      in_map_iter->second.stride_map(out_map_iter->second, out_tensor_strides);
-
-  const auto &ins = in_out.first;
-  const auto &outs = in_out.second;
-  // All size
-  const auto N = std::accumulate(ins.begin(), ins.end(), 0,
-                                 [](size_t curr_sum, const vector<int> &map) {
-                                   return curr_sum + map.size();
-                                 });
-
-  at::Tensor in_out_1 =
-      torch::empty({N}, torch::TensorOptions().dtype(torch::kInt64));
-  at::Tensor in_out_2 =
-      torch::empty({N}, torch::TensorOptions().dtype(torch::kInt64));
-
-  auto a_in_out_1 = in_out_1.accessor<long int, 1>();
-  auto a_in_out_2 = in_out_2.accessor<long int, 1>();
-
-  size_t curr_it = 0;
-  for (const auto &in : ins)
-    for (const auto i : in)
-      a_in_out_1[curr_it++] = i;
-
-  curr_it = 0;
-  for (const auto &out : outs)
-    for (const auto o : out)
-      a_in_out_2[curr_it++] = o;
-
-  return {in_out_1, in_out_2};
-}
-
-// Generate and return the ins -> out map.
-template <typename MapType>
-pair<vector<at::Tensor>, vector<at::Tensor>>
-CoordsManager<MapType>::getUnionMap(vector<py::object> py_in_coords_keys,
-                                    py::object py_out_coords_key) {
-
-  // all exception handling will be done inside the following
-  const InOutMapsRefPair<int> in_outs =
-      getUnionInOutMaps(py_in_coords_keys, py_out_coords_key);
-  const auto &ins = in_outs.first;
-  const auto &outs = in_outs.second;
-
-  // Size of the in out maps
-  const auto N = ins.size();
-
-  // Return torch tensor
-  vector<at::Tensor> th_ins;
-  vector<at::Tensor> th_outs;
-  for (size_t i = 0; i < N; ++i) {
-    at::Tensor th_in = torch::empty(
-        {(long)ins[i].size()}, torch::TensorOptions().dtype(torch::kInt64));
-    at::Tensor th_out = torch::empty(
-        {(long)outs[i].size()}, torch::TensorOptions().dtype(torch::kInt64));
-
-    copy_types(ins[i], th_in);
-    copy_types(outs[i], th_out);
-
-    th_ins.push_back(move(th_in));
-    th_outs.push_back(move(th_out));
-  }
-
-  return make_pair(th_ins, th_outs);
-}
-
-*/
 
 /*******************************
  * Initialization
@@ -189,7 +102,8 @@ struct insert_and_map_functor<coordinate_type, coordinate_field_type,
 template <typename coordinate_type, typename coordinate_field_type>
 struct insert_field_functor<
     coordinate_type, coordinate_field_type, std::allocator, CoordinateMapCPU,
-    CoordinateFieldMapCPU<coordinate_field_type, std::allocator>> {
+    CoordinateFieldMapCPU<coordinate_field_type, coordinate_type,
+                          std::allocator>> {
 
   void
   operator()(coordinate_map_key_type &map_key, at::Tensor const &th_coordinate,
@@ -200,8 +114,9 @@ struct insert_field_functor<
     uint32_t const coordinate_size = th_coordinate.size(1);
     coordinate_field_type *p_coordinate =
         th_coordinate.data_ptr<coordinate_field_type>();
-    auto map = CoordinateFieldMapCPU<coordinate_field_type, std::allocator>(
-        N, coordinate_size, map_key.first);
+    auto map = CoordinateFieldMapCPU<coordinate_field_type, coordinate_type,
+                                     std::allocator>(N, coordinate_size,
+                                                     map_key.first);
     map.insert(p_coordinate, p_coordinate + N * coordinate_size);
 
     LOG_DEBUG("insert map with tensor_stride", map_key.first);
@@ -270,15 +185,166 @@ py::object CoordinateMapManager<coordinate_type, coordinate_field_type,
   return py_key;
 }
 
+/* to_sparse_and_map */
+/*
+ * coords: coordinates in IntTensor
+ * tensor_strides: current tensor strides this coords will be initializeds
+ */
+template <typename coordinate_type, typename coordinate_field_type,
+          template <typename C> class TemplatedAllocator,
+          template <typename T, template <typename Q> class A>
+          class CoordinateMapType>
+std::pair<py::object, std::pair<at::Tensor, at::Tensor>>
+CoordinateMapManager<coordinate_type, coordinate_field_type, TemplatedAllocator,
+                     CoordinateMapType>::
+    field_to_sparse_insert_and_map(
+        CoordinateMapKey const *p_in_field_map_key,
+        default_types::stride_type const sparse_tensor_stride,
+        std::string const sparse_tensor_string_id) {
+  auto const coordinate_size = p_in_field_map_key->get_coordinate_size();
+  // Basic assertions
+  ASSERT(coordinate_size - 1 == sparse_tensor_stride.size(),
+         "The coordinate dimension (coordinate_size - 1):", coordinate_size - 1,
+         " must match the size of tensor stride:",
+         ArrToString(sparse_tensor_stride));
+
+  // Find coordinate field
+  auto const it = m_field_coordinates.find(p_in_field_map_key->get_key());
+  ASSERT(it != m_field_coordinates.end(), ERROR_MAP_NOT_FOUND);
+  auto const &field_map = it->second;
+
+  auto options = torch::TensorOptions().dtype(torch::kInt).requires_grad(false);
+
+  if (!detail::is_cpu_coordinate_map<CoordinateMapType>::value) {
+#ifndef CPU_ONLY
+    auto device_id = at::cuda::current_device();
+    options = options.device(torch::kCUDA, device_id);
+#else
+    ASSERT(false, ERROR_CPU_ONLY);
+#endif
+  }
+
+  // generate the map_key
+  coordinate_map_key_type map_key =
+      std::make_pair(sparse_tensor_stride, sparse_tensor_string_id);
+  if (m_coordinate_maps.find(map_key) != m_coordinate_maps.end()) {
+    LOG_DEBUG("CoordinateMapKey collision detected:", map_key,
+              "generating new string id.");
+    map_key =
+        get_random_string_id(sparse_tensor_stride, sparse_tensor_string_id);
+  }
+
+  LOG_DEBUG("initializing a field map with tensor stride:", map_key.first,
+            "string id:", map_key.second);
+
+  // Quantize the field with tensor stride.
+  // The coordinate must be a tensor. Wrap a pointer with a tensor.
+  at::Tensor int_coordinates =
+      at::empty({field_map.size(), coordinate_size}, options);
+  field_map.quantize_coordinates(int_coordinates.data_ptr<coordinate_type>(),
+                                 sparse_tensor_stride);
+
+  auto const map_inverse_map =
+      detail::insert_and_map_functor<coordinate_type, coordinate_field_type,
+                                     TemplatedAllocator, CoordinateMapType>()(
+          map_key, int_coordinates, *this);
+
+  auto const field_to_sparse_map_key =
+      std::pair<coordinate_map_key_type, coordinate_map_key_type>{
+          p_in_field_map_key->get_key(), map_key};
+
+  auto result = m_field_to_sparse_maps.insert(
+      std::pair<
+          const std::pair<coordinate_map_key_type, coordinate_map_key_type>,
+          const std::pair<at::Tensor, at::Tensor>>{field_to_sparse_map_key,
+                                                   map_inverse_map});
+  LOG_DEBUG("field to sparse tensor map insertion", result.second);
+
+  py::object py_key = py::cast(new CoordinateMapKey(coordinate_size, map_key));
+
+  return std::make_pair(py_key, map_inverse_map);
+}
+
+/*
+ * Return existing field to sparse map. Raise an except if the map doesn't
+ * exist
+ */
+template <typename coordinate_type, typename coordinate_field_type,
+          template <typename C> class TemplatedAllocator,
+          template <typename T, template <typename Q> class A>
+          class CoordinateMapType>
+std::pair<at::Tensor, at::Tensor>
+CoordinateMapManager<coordinate_type, coordinate_field_type, TemplatedAllocator,
+                     CoordinateMapType>::
+    get_field_to_sparse_map(CoordinateMapKey const *p_field_key,
+                            CoordinateMapKey const *p_sparse_key) const {
+  auto key = std::pair<coordinate_map_key_type, coordinate_map_key_type>{
+      p_field_key->get_key(), p_sparse_key->get_key()};
+  auto it = m_field_to_sparse_maps.find(key);
+  ASSERT(it != m_field_to_sparse_maps.end(),
+         "Field To Sparse Map doesn't exist");
+  return it->second;
+}
+
+/*
+ * Create a field to sparse map if it doesn't exist.
+ */
+template <typename coordinate_type, typename coordinate_field_type,
+          template <typename C> class TemplatedAllocator,
+          template <typename T, template <typename Q> class A>
+          class CoordinateMapType>
+std::pair<at::Tensor, at::Tensor>
+CoordinateMapManager<coordinate_type, coordinate_field_type, TemplatedAllocator,
+                     CoordinateMapType>::
+    field_to_sparse_map(CoordinateMapKey const *p_in_field_map_key,
+                        CoordinateMapKey const *p_out_sparse_map_key) {
+
+  auto const coordinate_size = p_in_field_map_key->get_coordinate_size();
+  // Basic assertions
+  ASSERT(coordinate_size == p_out_sparse_map_key->get_coordinate_size(),
+         "The coordinate dimension mismatch.", coordinate_size,
+         "!=", p_out_sparse_map_key->get_coordinate_size());
+
+  // Find coordinate field
+  auto const it_field = m_field_coordinates.find(p_in_field_map_key->get_key());
+  ASSERT(it_field != m_field_coordinates.end(), ERROR_MAP_NOT_FOUND);
+  auto const &field_map = it_field->second;
+  auto const it_sparse =
+      m_coordinate_maps.find(p_out_sparse_map_key->get_key());
+  ASSERT(it_sparse != m_coordinate_maps.end(), ERROR_MAP_NOT_FOUND);
+  auto const &sparse_map = it_sparse->second;
+
+  auto options = torch::TensorOptions().dtype(torch::kInt).requires_grad(false);
+
+  if (!detail::is_cpu_coordinate_map<CoordinateMapType>::value) {
+#ifndef CPU_ONLY
+    auto device_id = at::cuda::current_device();
+    options = options.device(torch::kCUDA, device_id);
+#else
+    ASSERT(false, ERROR_CPU_ONLY);
+#endif
+  }
+
+  auto const map_inverse_map =
+      sparse_map.field_map(field_map.const_coordinate_data(), field_map.size());
+
+  auto const field_to_sparse_map_key =
+      std::pair<coordinate_map_key_type, coordinate_map_key_type>{
+          p_in_field_map_key->get_key(), p_out_sparse_map_key->get_key()};
+
+  auto result = m_field_to_sparse_maps.insert(
+      std::pair<
+          const std::pair<coordinate_map_key_type, coordinate_map_key_type>,
+          const std::pair<at::Tensor, at::Tensor>>{field_to_sparse_map_key,
+                                                   map_inverse_map});
+  LOG_DEBUG("field to sparse tensor map insertion", result.second);
+
+  return map_inverse_map;
+}
+
 /*
  * coords: coordinates in IntTensor
- * mapping: output mapping in IntTensor
  * tensor_strides: current tensor strides this coords will be initializeds
- * force_creation: even when there's a duplicate coords with the same tensor
- *                 strides.
- * force_remap: if there's duplicate coords, remap
- * allow_duplicate_coords: create map when there are duplicates in the
- * coordinates
  */
 template <typename coordinate_type, typename coordinate_field_type,
           template <typename C> class TemplatedAllocator,
@@ -438,6 +504,48 @@ CoordinateMapManager<coordinate_type, coordinate_field_type, TemplatedAllocator,
   return std::make_pair(origin_map_key, !exists_origin_map);
 }
 
+template <typename coordinate_type, typename coordinate_field_type,
+          template <typename C> class TemplatedAllocator,
+          template <typename T, template <typename Q> class A>
+          class CoordinateMapType>
+std::pair<coordinate_map_key_type, bool>
+CoordinateMapManager<coordinate_type, coordinate_field_type, TemplatedAllocator,
+                     CoordinateMapType>::origin_field() {
+  ASSERT(m_field_coordinates.size() > 0, "No coordinate map found");
+  // check if the key exists.
+  field_map_type const &random_map = m_field_coordinates.begin()->second;
+  stride_type origin_tensor_stride(random_map.coordinate_size() - 1);
+  std::for_each(origin_tensor_stride.begin(), origin_tensor_stride.end(),
+                [](auto &i) { i = 0; });
+  LOG_DEBUG("origin tensor stride:", origin_tensor_stride);
+
+  coordinate_map_key_type origin_map_key(origin_tensor_stride, "");
+  bool const exists_origin_map = exists(origin_map_key);
+
+  if (!exists_origin_map) {
+    LOG_DEBUG("origin coordinate map not found");
+    field_map_type const *p_min_coordinate_map{nullptr};
+    size_type min_size = std::numeric_limits<size_type>::max();
+    for (auto map_it = m_field_coordinates.begin();
+         map_it != m_field_coordinates.end(); ++map_it) {
+      if (min_size > map_it->second.size()) {
+        p_min_coordinate_map = &(map_it->second);
+      }
+    }
+
+    if (p_min_coordinate_map != nullptr) {
+      map_type origin_map = p_min_coordinate_map->origin();
+      LOG_DEBUG("origin map with size:", origin_map.size(), " inserted");
+      insert(origin_map_key, origin_map);
+    } else {
+      ASSERT(false, "Invalid origin map");
+    }
+  }
+
+  // (key, new map generated flag)
+  return std::make_pair(origin_map_key, !exists_origin_map);
+}
+
 template <typename coordinate_type, typename coordinate_field_type,
           template <typename C> class TemplatedAllocator,
           template <typename T, template <typename Q> class A>
@@ -448,18 +556,15 @@ CoordinateMapManager<coordinate_type, coordinate_field_type, TemplatedAllocator,
                                                    &in_key,
                                                bool const *keep_begin,
                                                bool const *keep_end) {
-
-  ASSERT(exists(in_key), "In map doesn't exist");
-
-  // create a coordinate_map_key
-  coordinate_map_key_type map_key = std::make_pair(in_key.first, "");
-  if (m_coordinate_maps.find(map_key) != m_coordinate_maps.end()) {
-    map_key = get_random_string_id(map_key.first, "");
-  }
-
   auto const map_it = m_coordinate_maps.find(in_key);
   ASSERT(map_it != m_coordinate_maps.end(), ERROR_MAP_NOT_FOUND);
 
+  // create a coordinate_map_key
+  coordinate_map_key_type map_key = std::make_pair(in_key.first, "pruned");
+  if (m_coordinate_maps.find(map_key) != m_coordinate_maps.end()) {
+    map_key = get_random_string_id(map_key.first, map_key.second);
+  }
+
   map_type pruned_map = map_it->second.prune(keep_begin, keep_end);
   LOG_DEBUG("pruned map with size:", pruned_map.size(), " inserted");
   insert(map_key, pruned_map);
@@ -504,6 +609,13 @@ template <> struct swap_in_out_map_functor<cpu_kernel_map> {
   }
 };
 
+template <typename coordinate_type>
+struct empty_map_functor<coordinate_type, std::allocator, CoordinateMapCPU,
+                         cpu_kernel_map> {
+
+  cpu_kernel_map operator()() { return cpu_kernel_map{}; }
+};
+
 } // namespace detail
 
 /*
@@ -596,6 +708,13 @@ CoordinateMapManager<
     // +1 for batch index
     ASSERT(kernel_dim + 1 == in_map.coordinate_size(), "kernel size mismatch");
     ASSERT(kernel_dim + 1 == out_map.coordinate_size(), "kernel size mismatch");
+
+    // If either coordinate map is empty
+    if (in_map.size() == 0 || out_map.size() == 0) {
+      return detail::empty_map_functor<coordinate_type, TemplatedAllocator,
+                                       CoordinateMapType, kernel_map_type>()();
+    }
+
     if (!is_transpose) {
       if (is_pool && (kernel_stride == kernel_size)) {
         LOG_DEBUG("generating stride_map");
@@ -610,6 +729,13 @@ CoordinateMapManager<
         LOG_DEBUG("generating kernel map");
 
         // Default kernel map
+        LOG_DEBUG(
+            "kernel region with kernel: ",
+            PtrToString(kernel_size.data(), in_map.coordinate_size() - 1));
+        LOG_DEBUG(
+            "kernel region with dilation: ",
+            PtrToString(kernel_dilation.data(), in_map.coordinate_size() - 1));
+
         auto kernel_region = cpu_kernel_region<coordinate_type>(
             region_type,                       //
             in_map.coordinate_size(),          //
@@ -736,7 +862,7 @@ struct origin_map_functor<coordinate_type, std::allocator, CoordinateMapCPU,
          ++out_row_index) {
       auto const &in_map = origin_map.first[out_row_index];
       int32_t const curr_size = in_map.size();
-      ASSERT(curr_size > 0, "invalid kernel map");
+      ASSERT(curr_size > 0, "invalid kernel map for index", out_row_index);
       auto const curr_batch_index = p_batch_indices[out_row_index];
 
       ASSERT(curr_batch_index <= max_batch_index, "invalid batch index");
@@ -783,6 +909,127 @@ CoordinateMapManager<coordinate_type, coordinate_field_type, TemplatedAllocator,
   return m_kernel_maps[kernel_map_key];
 }
 
+template <typename coordinate_type, typename coordinate_field_type,
+          template <typename C> class TemplatedAllocator,
+          template <typename T, template <typename Q> class A>
+          class CoordinateMapType>
+typename CoordinateMapManager<coordinate_type, coordinate_field_type,
+                              TemplatedAllocator,
+                              CoordinateMapType>::kernel_map_type const &
+CoordinateMapManager<coordinate_type, coordinate_field_type, TemplatedAllocator,
+                     CoordinateMapType>::origin_field_map(CoordinateMapKey const
+                                                              *p_in_map_key) {
+  ASSERT(exists_field(p_in_map_key), ERROR_MAP_NOT_FOUND);
+  kernel_map_key_type const kernel_map_key =
+      origin_map_key(p_in_map_key->get_key());
+  coordinate_map_key_type const origin_key = std::get<1>(kernel_map_key);
+
+  if (m_field_kernel_maps.find(kernel_map_key) == m_field_kernel_maps.end()) {
+    auto const key = origin_field().first;
+    auto const &origin_coordinate_map = m_coordinate_maps.find(key)->second;
+    auto origin_map = m_field_coordinates.find(p_in_map_key->get_key())
+                          ->second.origin_map(origin_coordinate_map);
+    m_field_kernel_maps[kernel_map_key] = std::move(origin_map);
+  }
+
+  return m_field_kernel_maps[kernel_map_key];
+}
+namespace detail {
+
+template <typename coordinate_type>
+struct stride_map2tensor_functor<coordinate_type, std::allocator,
+                                 CoordinateMapCPU, cpu_kernel_map> {
+
+  std::pair<at::Tensor, at::Tensor>
+  operator()(cpu_kernel_map const &stride_kernel_map) {
+
+    ASSERT(stride_kernel_map.first.size() == 1, "Invalid kernel_map");
+    ASSERT(stride_kernel_map.first.size() == stride_kernel_map.second.size(),
+           "invalid kernel_map");
+
+    auto const &in_map = stride_kernel_map.first[0];
+    auto const &out_map = stride_kernel_map.second[0];
+
+    auto options =
+        torch::TensorOptions().dtype(torch::kLong).requires_grad(false);
+    int64_t const out_size = (int64_t)in_map.size();
+
+    at::Tensor th_in_map = torch::empty({out_size}, options);
+    at::Tensor th_out_map = torch::empty({out_size}, options);
+    int64_t *p_in_map = th_in_map.data_ptr<int64_t>();
+    int64_t *p_out_map = th_out_map.data_ptr<int64_t>();
+
+    // from int32_t to long type
+    for (uint32_t i = 0; i < out_size; ++i)
+      p_in_map[i] = in_map[i];
+    for (uint32_t i = 0; i < out_size; ++i)
+      p_out_map[i] = out_map[i];
+
+    return std::make_pair(std::move(th_in_map), std::move(th_out_map));
+  }
+};
+
+} // namespace detail
+
+template <typename coordinate_type, typename coordinate_field_type,
+          template <typename C> class TemplatedAllocator,
+          template <typename T, template <typename Q> class A>
+          class CoordinateMapType>
+std::pair<at::Tensor, at::Tensor>
+CoordinateMapManager<coordinate_type, coordinate_field_type, TemplatedAllocator,
+                     CoordinateMapType>::stride_map_th(CoordinateMapKey const
+                                                           *p_in_map_key,
+                                                       CoordinateMapKey const
+                                                           *p_strided_map_key) {
+
+  ASSERT(exists(p_in_map_key), ERROR_MAP_NOT_FOUND);
+  ASSERT(exists(p_strided_map_key), ERROR_MAP_NOT_FOUND);
+
+  map_type const &in_map =
+      m_coordinate_maps.find(p_in_map_key->get_key())->second;
+  map_type const &strided_map =
+      m_coordinate_maps.find(p_strided_map_key->get_key())->second;
+
+  // Get tensor strides and find kernel stride size
+  // Check if the kernel map key exists
+  auto const &in_map_stride = in_map.get_tensor_stride();
+  auto const &strided_map_stride = strided_map.get_tensor_stride();
+
+  stride_type kernel_stride(in_map_stride.size());
+  for (index_type i = 0; i < kernel_stride.size(); ++i) {
+    ASSERT(strided_map_stride[i] % in_map_stride[i] == 0,
+           "The tensor stride of the strided map must be divisible by the "
+           "tensor stride of the input map. strided_map_stride:",
+           ArrToString(strided_map_stride),
+           " in_map_stride:", ArrToString(in_map_stride));
+    kernel_stride[i] = strided_map_stride[i] / in_map_stride[i];
+  }
+
+  auto const one_vec = detail::ones(in_map.coordinate_size() - 1);
+  kernel_map_key_type const kernel_map_key = std::make_tuple(
+      p_in_map_key->get_key(), p_strided_map_key->get_key(), // maps
+      kernel_stride, kernel_stride, one_vec,                 // kernels
+      RegionType::HYPER_CUBE /* region_type */, 0 /* is_transpose */,
+      true /* is_pool */);
+
+  if (m_kernel_maps.find(kernel_map_key) == m_kernel_maps.end()) {
+    LOG_DEBUG("Creating stride kernel map with kernel size:",
+              ArrToString(kernel_stride));
+    auto const stride_map =
+        detail::stride_map_functor<coordinate_type, TemplatedAllocator,
+                                   CoordinateMapType, kernel_map_type>()(
+            in_map, strided_map, kernel_stride);
+
+    m_kernel_maps[kernel_map_key] = std::move(stride_map);
+  }
+
+  // copy the kernel map to tensors
+  return detail::stride_map2tensor_functor<coordinate_type, TemplatedAllocator,
+                                           CoordinateMapType,
+                                           kernel_map_type>()(
+      m_kernel_maps[kernel_map_key]);
+}
+
 template <typename coordinate_type, typename coordinate_field_type,
           template <typename C> class TemplatedAllocator,
           template <typename T, template <typename Q> class A>
@@ -801,6 +1048,24 @@ CoordinateMapManager<coordinate_type, coordinate_field_type, TemplatedAllocator,
       origin_map, kernel_map);
 }
 
+template <typename coordinate_type, typename coordinate_field_type,
+          template <typename C> class TemplatedAllocator,
+          template <typename T, template <typename Q> class A>
+          class CoordinateMapType>
+std::pair<at::Tensor, std::vector<at::Tensor>>
+CoordinateMapManager<coordinate_type, coordinate_field_type, TemplatedAllocator,
+                     CoordinateMapType>::
+    origin_field_map_th(CoordinateMapKey const *p_in_map_key) {
+  kernel_map_type const &kernel_map = origin_field_map(p_in_map_key);
+
+  coordinate_map_key_type const origin_key = origin_field().first;
+  map_type const &origin_map = m_coordinate_maps.find(origin_key)->second;
+
+  return detail::origin_map_functor<coordinate_type, TemplatedAllocator,
+                                    CoordinateMapType, kernel_map_type>()(
+      origin_map, kernel_map);
+}
+
 // Interpolation map
 template <typename coordinate_type, typename coordinate_field_type,
           template <typename C> class TemplatedAllocator,
@@ -1121,14 +1386,14 @@ template <typename coordinate_type, typename coordinate_field_type,
           class CoordinateMapType>
 std::unordered_map<int64_t, at::Tensor> CoordinateMapManager<
     coordinate_type, coordinate_field_type, TemplatedAllocator,
-    CoordinateMapType>::get_kernel_map(CoordinateMapKey const *p_in_map_key,
-                                       CoordinateMapKey const *p_out_map_key,
-                                       stride_type const &kernel_size, //
-                                       stride_type const &kernel_stride,
-                                       stride_type const &kernel_dilation,
-                                       RegionType::Type const region_type,
-                                       at::Tensor const &offset,
-                                       bool is_transpose, bool is_pool) {
+    CoordinateMapType>::kernel_map_th(CoordinateMapKey const *p_in_map_key,
+                                      CoordinateMapKey const *p_out_map_key,
+                                      stride_type const &kernel_size, //
+                                      stride_type const &kernel_stride,
+                                      stride_type const &kernel_dilation,
+                                      RegionType::Type const region_type,
+                                      at::Tensor const &offset,
+                                      bool is_transpose, bool is_pool) {
 
   auto const &curr_kernel_map =
       kernel_map(p_in_map_key, p_out_map_key,                 // maps
